function [Z,U,CPrank,w,RankChange,Wchange] = TREL1_CP(T, Omega,mu, maxIter, epsilon,R)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Created by Qiquan on 05/11/2015
% Revised by Qiquan on 08/04/2015
% Revised by Qiquan on 09/09/2016
sizeT = size(T);
X = T;
ninit=2;
nitr=7;
[S1 S2 S3] = size(T);
Weights_L= zeros(maxIter, 1);
Wchange=zeros(maxIter, R);
RankChange=zeros(maxIter, 1);

% initialization by Robust Tensor Power Method (modified for non-symmetric tensors)
U01 = zeros(S1,R);
U02 = zeros(S2,R);
U03 = zeros(S3,R);
S0 = zeros(R,1);
for i0=1:R
    tU1 = zeros(S1,ninit);
    tU2 = zeros(S2,ninit);
    tU3 = zeros(S3,ninit);
    tS  = zeros(ninit,1);
    for init=1:ninit
        [tU1(:,init) tU2(:,init) tU3(:,init)] = RTPM(T-CPcomp(S0,U01,U02,U03), nitr);
        tU1(:,init) = tU1(:,init)./norm(tU1(:,init));
        tU2(:,init) = tU2(:,init)./norm(tU2(:,init));
        tU3(:,init) = tU3(:,init)./norm(tU3(:,init));
        tS(init) = TenProj(T-CPcomp(S0,U01,U02,U03),tU1(:,init),tU2(:,init),tU3(:,init) );
    end
    [C I] = max(tS);
    U01(:,i0) = tU1(:,I)/norm(tU1(:,I));
    U02(:,i0) = tU2(:,I)/norm(tU2(:,I));
    U03(:,i0) = tU3(:,I)/norm(tU3(:,I));
    S0(i0) = TenProj(T-CPcomp(S0,U01,U02,U03),U01(:,i0),U02(:,i0),U03(:,i0));
end
% apply alternating least squares (or equivalently alternating minimization)
U{1} = U01;   U{2} = U02;   U{3} = U03;
w= S0;
w_old=w;

%% Iteration Scheme
for k = 1: maxIter
    
    
    Xr =  X;
    % Updating U{i} and G
    
    for r= 1 : R
        
        if w(r) ~= 0
            index0=find(abs(w(1:r))>0);
            R0=length(index0);
            Xrn=ttm(tensor(Xr),{U{2}(:,r),U{3}(:,r)}, [2 3], 't');
            u1=zeros(sizeT(1),1);
            for q=1:R0-1
                u1=u1+U{1}(:,index0(q))'*double(Xrn(:))*U{1}(:,index0(q));
            end
            midU1 =(double(Xrn(:))- u1)/(w(r));
            if   norm(midU1(:)) ~= 0
                U{1}(:,r)= midU1/norm(midU1(:));
                
                Xrn=ttm(tensor(Xr),{U{1}(:,r),U{3}(:,r)}, [1 3], 't');
                u2=zeros(sizeT(2),1);
                for q=1:R0-1
                    u2=u2 + U{2}(:,index0(q))'*double(Xrn(:))*U{2}(:,index0(q));
                end
                midU2 =(u2-double(Xrn(:)) )/(w(r));
                if   norm(midU2(:)) ~= 0
                    U{2}(:,r)= midU2/norm(midU2(:));
                    Xrn=ttm(tensor(Xr),{U{1}(:,r),U{2}(:,r)}, [1 2], 't');
                    u3=zeros(sizeT(3),1);
                    for q=1:R0-1
                        u3=u3 + U{3}(:,index0(q))'*double(Xrn(:))*U{3}(:,index0(q));
                    end
                    midU3 =(double(Xrn(:))-u3)/(w(r));
                    if  norm(midU3(:)) ~= 0
                        U{3}(:,r)= midU3/norm(midU3(:));
                        midw = outerprod([{U{1}(:,r)},{U{2}(:,r)},{U{3}(:,r)}]);
                        DO=dot(Xr(:),midw(:));
                        w(r) = soft(DO,mu);
                        Weights_L(k,r)=w(r);
                        Xr =  Xr - w(r)*midw;  % Residual of approximation
                    end
                end
            end
        end
    end
    Z =  X -Xr;            % Approximation Z aims to finally equal to ground truth M;
    X(~Omega) = Z(~Omega); % Update the missing part in X using approximated vaules in Z;
    
    %% Check the erro  norm(X(:) - Z(:))^2
    RankChange(k,1)=length(find(abs(w)/sum(abs(w))>eps));
%     length(find((w)>eps));
    Wchange(k,1)=norm(abs(w)-abs(w_old))/norm(abs(w_old));
    w_old=w;
    if   mod(k+1, 5) == 0
        if Wchange(k,1) < epsilon
            break;
        end
    end
end
%% Estimate the real rank of the incomplete matrix X
%     if find(w==0)
index1=find(w/sum(w)>eps);
Noabs =length(index1);
index2=find(abs(w)/sum(abs(w))>eps);
ABSw =length(index2);
fprintf('********  size(find(w>0)) = %d \n \n ',length(find(w>0)));
if  Noabs > ABSw
    R=ABSw;
    w=w(index2);
    index=index2;
else
    R=Noabs;
    w=w(index1);
    index=index1;
end
maxR=R;
% U10=zeros(sizeT(1),maxR);
% U20=zeros(sizeT(2),maxR);
% U30=zeros(sizeT(2),maxR);
fprintf('******** L1MC-RF estimated rank = %d, iterations = %d******** \n \n ', maxR,k);
U10=U{1}(:,index(1:R));
U20=U{2}(:,index(1:R));
U30=U{3}(:,index(1:R));

U{1}=U10;
U{2}=U20;
U{3}=U30;


    function T = CPcomp(S,U1,U2,U3)
        n1 = size(U1,1);n2 = size(U2,1);n3 = size(U3,1);
        s = min([length(S) size(U1,2) size(U2,2) size(U3,2)]);
        T = zeros(n1,n2,n3);
        for tt=1:n3
            T(:,:,tt) = U1*diag(U3(tt,:).*S(:)')*U2';
        end
    end
CPrank= R;
    function y = soft(x,T)
        if sum(abs(T(:)))==0
            y = x;
        else
            y = max(abs(x) - T, 0);
            y = sign(x).*y;
        end
    end


    function M = TenProj(T,U1,U2,U3)
        n1=size(U1,1);n2=size(U2,1);n3=size(U3,1);
        r1=size(U1,2);r2=size(U2,2);r3=size(U3,2);
        M =zeros(r1,r2,r3);
        for i=1:r3
            A = zeros(n1,n2);
            for j=1:n3
                A = A+T(:,:,j)*U3(j,i);
            end
            M(:,:,i) = U1'*A*U2;
        end
    end

    function [u1 u2 u3] = RTPM(T, mxitr)
        n1=size(T, 1);n2=size(T, 2);n3=size(T, 3);
        uinit=randn(n1,1);u1=uinit/norm(uinit);
        uinit=randn(n2,1);u2=uinit/norm(uinit);
        uinit=randn(n3,1);u3=uinit/norm(uinit);
        for itr=1:mxitr
            v1=zeros(n1,1);v2=zeros(n2,1);v3=zeros(n3,1);
            for i3=1:n3
                v3(i3)=u1'*T(:, :, i3)*u2;
                v1 = v1 + u3(i3)*T(:, :, i3)*u2;
                v2 = v2 + u3(i3)*T(:, :, i3)'*u1;
            end
            u10 = u1;
            u1 = v1/norm(v1);
            u20 = u2;
            u2 = v2/norm(v2);
            u30 = u3;
            u3 = v3/norm(v3);
            if(norm(u10-u1)+norm(u20-u2)+norm(u30-u3)<1e-7) break; end
        end
    end
%finalCP=CPcomp(w,U{1}(:,1:R),U{2}(:,1:R),U{3}(:,1:R));
end