function G = Update_G(U,X,Y,mu)
%% Closed form solution
% z = (ita*V*V^T + 2*gamma*I)^(-1) * V * (ita*x+y), where V=kron(V_1,...,V_n)
% -- 
% -- simplification
%
% let V_i*(V_i)^T = P_i*S_i*(P_i)^T, by spectral decomposition
% (ita*V*V^T + 2*gamma*I)^(-1) = 
%      kron(P_1,...,P_n) * (ita*kron(S_1,..S_n) + 2*gamma*I)^(-1) * kron((P_1)^T,...,(P_n)^T)
%     , where
%      (ita*kron(S_1,..S_n) + 2*gamma*I)^(-1) can be caculated efficiently

%%
wt = 1;
N = numel(U);

for i = ndims(X) : -1 : N+1
    wt = ones(size(X,i),1)*wt';
    wt = wt(:);
end
for i = N : -1 : 1
    [midU,S] = eig(U{i}*U{i}');
    Q{i} = S;
    P{i} = midU;
    wt = diag(S)*wt';
    wt = wt(:);
end
wt = reshape(mu*wt,size(X));  % diagonal vector of (ita*kron(S_1,..S_n) + 2*gamma*I)^(-1)
G = TensorChainProduct(mu*X+Y,U,1:N)-ones(numel(G),1); % ONE=ones(prod(size(Z)))
G = TensorChainProductT(G,P,1:N);
G = G./wt;
G = TensorChainProduct(G,P,1:N);