%% Construct a tensor with an underlying CP structure
sz = [10 5 3];
R  = 2;

for i=1:length(sz)
    A{i}  = randn(sz(i),R);
    for r = 1:R
      A{i}(:,r)= A{i}(:,r)/norm(A{i}(:,r));
    end
end
lambda = ones(R,1);

X     = full(ktensor(lambda,A));
Xorig = X;
%% Pick some random entries and set them to missing
M = round(rand(sz));
X(find(M==0))=NaN;

%% Suppose you are given the incomplete tensor X and you want to find the underlying CP components

% Record the places of the missing entries
W = tensor(~isnan(X.data));

% Set those missing entries in the tensor to 0 
X(find(W==0))=0;

% parameters for the gradient-based optimization algorithm
options = ncg('defaults');
options.MaxFuncEvals = 10000;
options.MaxIters     = 10000; % change this line to change the max number of iterations
options.StopTol      = 1e-6;
options.RelFuncTol   = 1e-6;
options.DisplayIters = 50;
[P, P0, output] = cp_wopt(X,W,R,'alg', 'ncg','alg_options',options,'init','nvecs');

% Compare P(factors extracted by the model) with A (true factors)
for i=1:3
    corr(P.U{i},A{i})
end
%% Reconstruct your data from the factor matrices
Xhat = full(P);
% Plot original vs. estimated values for the missing data
plot(Xorig(find(W==0)), Xhat(find(W==0)),'*');
xlabel('Original'); ylabel('Estimated')